@@ -18,7 +18,7 @@
    The GNU General Public License is often shipped with GNU software, and
    is generally kept in a file called COPYING or LICENSE.  If you do not
    have a copy of the license, write to the Free Software Foundation,
-   59 Temple Place, Suite 330, Boston, MA 02111 USA. */
+   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */
 
 #define READLINE_LIBRARY
 
@@ -855,7 +855,7 @@ _rl_read_init_file (filename, include_level)
 {
   register int i;
   char *buffer, *openname, *line, *end;
-  size_t file_size;
+  size_t file_size = 0;
 
   current_readline_init_file = filename;
   current_readline_init_include_level = include_level;
