@@ -1,4 +1,4 @@
-/* Copyright (C) 2000,2004 MySQL AB
+/* Copyright (c) 2000, 2011, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -11,7 +11,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
 
 /* This file should be included when using heap_database_functions */
 /* Author: Michael Widenius */
@@ -30,7 +30,7 @@ extern "C" {
 #include <thr_lock.h>
 #endif
 
-#include "my_handler.h"
+#include "my_compare.h"
 #include "my_tree.h"
 
 	/* defines used by heap-funktions */
@@ -50,6 +50,7 @@ typedef struct st_heapinfo		/* Struct from heap_info */
   uint reclength;			/* Length of one record */
   int errkey;
   ulonglong auto_increment;
+  time_t create_time;
 } HEAPINFO;
 
 
@@ -146,6 +147,7 @@ typedef struct st_heap_share
   uint open_count;
   uchar *del_link;			/* Link to next block with del. rec */
   char * name;			/* Name of "memory-file" */
+  time_t create_time;
 #ifdef THREAD
   THR_LOCK lock;
   pthread_mutex_t intern_lock;		/* Locking for use with _locking */
