@@ -34,6 +34,10 @@ trx_purge_t*	purge_sys = NULL;
 which needs no purge */
 trx_undo_rec_t	trx_purge_dummy_rec;
 
+#ifdef UNIV_DEBUG
+my_bool		srv_purge_view_update_only_debug;
+#endif /* UNIV_DEBUG */
+
 /*********************************************************************
 Checks if trx_id is >= purge_view: then it is guaranteed that its update
 undo log still exists in the system. */
@@ -209,6 +213,7 @@ trx_purge_sys_create(void)
 	purge_sys->purge_trx_no = ut_dulint_zero;
 	purge_sys->purge_undo_no = ut_dulint_zero;
 	purge_sys->next_stored = FALSE;
+	ut_d(purge_sys->done_trx_no = ut_dulint_zero);
 
 	rw_lock_create(&purge_sys->latch, SYNC_PURGE_LATCH);
 
@@ -576,6 +581,7 @@ trx_purge_truncate_if_arr_empty(void)
 	ut_ad(mutex_own(&(purge_sys->mutex)));
 
 	if (purge_sys->arr->n_used == 0) {
+		ut_d(purge_sys->done_trx_no = purge_sys->purge_trx_no);
 
 		trx_purge_truncate_history();
 
@@ -1022,7 +1028,7 @@ trx_purge(void)
 {
 	que_thr_t*	thr;
 	/*	que_thr_t*	thr2; */
-	ulint		old_pages_handled;
+	ulonglong	old_pages_handled;
 
 	mutex_enter(&(purge_sys->mutex));
 
@@ -1077,6 +1083,13 @@ trx_purge(void)
 
 	rw_lock_x_unlock(&(purge_sys->latch));
 
+#ifdef UNIV_DEBUG
+	if (srv_purge_view_update_only_debug) {
+		mutex_exit(&(purge_sys->mutex));
+		return(0);
+	}
+#endif
+
 	purge_sys->state = TRX_PURGE_ON;
 
 	/* Handle at most 20 undo log pages in one purge batch */
@@ -1116,7 +1129,7 @@ trx_purge(void)
 			(ulong) purge_sys->n_pages_handled);
 	}
 
-	return(purge_sys->n_pages_handled - old_pages_handled);
+	return((ulint) (purge_sys->n_pages_handled - old_pages_handled));
 }
 
 /**********************************************************************
