@@ -1,4 +1,5 @@
-/* Copyright (c) 2007 MySQL AB, 2009 Sun Microsystems, Inc.
+/*
+   Copyright (c) 2007 MySQL AB, 2009 Sun Microsystems, Inc.
    Use is subject to license terms.
 
    This program is free software; you can redistribute it and/or modify
@@ -12,7 +13,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+*/
 
 #include <my_global.h>
 #include <my_sys.h>
@@ -73,4 +75,5 @@ int main (int argc, char **argv)
   my_end(0);
 
   return 0;
-};
+}
+
